#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include "windows.h"
#endif

#include "stdio.h"
#include "math.h"
#include "stdlib.h"
#include "string.h"

#include "GL/gl.h"
#include "SDL.h"
#include "SDL_mixer.h"

#include "BList.h"
#include "Symbol.h"
#include "2DCMC.h"
#include "auxiliar.h"
#include "GLTile.h"
#include "2DCMC.h"
#include "keyboardstate.h"
#include "randomc.h"

#include "GLTManager.h"
#include "SoundManager.h"
#include "SFXManager.h"
#include "MusicManager.h"
#include "KObject.h"
#include "Knightmare.h"
#include "KnightmareApp.h"

#include "Binterface.h"

#include "debug.h"

extern bool fullscreen;
extern SDL_Surface *screen_sfc;

extern SDL_Surface *toogle_video_mode(bool fullscreen);

int KnightmareApp::title_cycle(KEYBOARDSTATE *k)
{
	int old_mouse_x = m_mouse_x;
	int old_mouse_y = m_mouse_y;

	SDL_GetMouseState(&m_mouse_x,&m_mouse_y);
	if (!m_mouse_click_x.EmptyP()) {
		m_mouse_click_x.Delete();
		m_mouse_click_y.Delete();
		m_mouse_button=1;
	} else {
		m_mouse_button=0;
	} // if

	if (m_mouse_x==old_mouse_x && m_mouse_y==old_mouse_y) {
		m_mouse_movement_timmer++;

		if (m_mouse_movement_timmer>150 && m_draw_mouse_status==2) {
			m_draw_mouse_status=3;
			m_draw_mouse=1.0f;
		} // if
	} else {
		m_mouse_movement_timmer=0;

		if (m_draw_mouse_status==0) {
			m_draw_mouse_status=1;
			m_draw_mouse=0.0f;
		} // if
	} // if

	switch(m_draw_mouse_status) {
	case 0:
			break;
	case 1: m_draw_mouse+=0.05f;
			if (m_draw_mouse>=1.0f) {
				m_draw_mouse=1.0f;
				m_draw_mouse_status=2;
			} // if
			break;
	case 2:
			break;
	case 3: m_draw_mouse-=0.05f;
			if (m_draw_mouse<=0.0f) {
				m_draw_mouse=0.0f;
				m_draw_mouse_status=0;
			} // if
			break;
	} // switch

	if (m_game!=0) {
		delete m_game;
		m_game=0;
		m_SFXM->SFX_stop_continuous();
	} // if

	if (m_state_cycle==0) {
		m_MusicM->music_play(0, "title", m_music_volume,-1);
		getMenu(MENU_MAIN);	// Creates the initial menu
		m_title_menu_state = 0;
		m_title_entry_effect = 0.0f;
		m_title_menu_selected = 0;
	}

	// FADE IN:
	if (m_title_menu_state==0) {
		m_title_entry_effect+=0.05f;
		if (m_title_entry_effect>=1.0f) {
			m_title_menu_state = 1;
		}
	}
	// FADE OUT:
	if (m_title_menu_state==2) {
		m_title_entry_effect-=0.05f;
		if (m_title_entry_effect<=0.0f) {
			m_title_menu_state = 3;

			switch(m_title_menu_action) {
			case 0: m_n_players = 1;
					m_current_level = 1;
					m_n_lives_left_player1 = 2;
					m_current_score = 0;
					return STATE_GAMESTART;
					break;
			case 1: m_n_players = 2;
					m_n_lives_left_player1 = 2;
					m_n_lives_left_player2 = 2;
					m_current_level = 1;
					m_current_score = 0;
					return STATE_GAMESTART;
					break;
			case 2:
					break;
			case 3:
					break;
			case 4:
					getMenu(MENU_OPTIONS);
					m_title_menu_state = 0;
					m_title_entry_effect = 0.0f;
					m_title_menu_selected = 0;
					break;
			case 5: return STATE_NONE;
					break;
			case 100:
					getMenu(MENU_INPUT1);	
					m_title_menu_state = 0;
					m_title_entry_effect = 0.0f;
					m_title_menu_selected = 0;
					break;
			case 101:
					getMenu(MENU_INPUT2);	
					m_title_menu_state = 0;
					m_title_entry_effect = 0.0f;
					m_title_menu_selected = 0;
					break;
			case 105:
					getMenu(MENU_MAIN);	
					m_title_menu_state = 0;
					m_title_entry_effect = 0.0f;
					m_title_menu_selected = 0;
					break;
			case 208:
			case 308:
					getMenu(MENU_OPTIONS);	
					m_title_menu_state = 0;
					m_title_entry_effect = 0.0f;
					m_title_menu_selected = 0;
					break;
			}
		}
	}

	if (m_title_menu_state==0 || m_title_menu_state==1) {
		if (m_title_menu_redefining_key_player!=-1 && m_title_menu_redefining_key!=-1) {
			int i;
			for(i=0;i<SDLK_LAST;i++) {
				if (k->key_single_press(i)) {
					output_debug_message("Key %i of player %i redefined to %i\n",m_title_menu_redefining_key,
										 m_title_menu_redefining_key_player,i);
					if (m_title_menu_redefining_key_player==1) {
						m_keys_configuration1[m_title_menu_redefining_key] = i;
						m_title_menu_redefining_key_player = -1;
						m_title_menu_redefining_key = -1;
                        save_configuration();
						getMenu(MENU_INPUT1);
					} else {
						m_keys_configuration2[m_title_menu_redefining_key] = i;
						m_title_menu_redefining_key_player = -1;
						m_title_menu_redefining_key = -1;
                        save_configuration();
						getMenu(MENU_INPUT2);
					}
				}
			}
		} else {
			if (keyPress(KKEY_DOWN,-1,k)) {
				m_title_menu_selected=(m_title_menu_selected+1)%m_menu.Length();
				 m_SFXM->SFX_play("sfx/cursormove", m_sfx_volume);
			}
			if (keyPress(KKEY_UP,-1,k)) {
				m_title_menu_selected--;
				if (m_title_menu_selected<0) m_title_menu_selected=m_menu.Length()-1;
				 m_SFXM->SFX_play("sfx/cursormove", m_sfx_volume);
			}
			if (keyPress(KKEY_LEFT,-1,k)) {
				switch(m_menu[m_title_menu_selected]->m_ID) {
					case 102: // change music volume down
						if (m_music_volume==0) {
							m_music_volume=MIX_MAX_VOLUME;
						} else {
							m_music_volume-=32;
							if (m_music_volume<0) m_music_volume = 0;
						}
						save_configuration();
						getMenu(MENU_OPTIONS);
						m_SFXM->SFX_play("sfx/cursormove", m_sfx_volume);
						m_MusicM->music_volume(0, m_music_volume);
						break;
					case 103: // change sfx volume up
						if (m_sfx_volume==0) {
							m_sfx_volume=MIX_MAX_VOLUME;
						} else {
							m_sfx_volume-=32;
							if (m_sfx_volume<0) m_music_volume = 0;
						}
						save_configuration();
						getMenu(MENU_OPTIONS);
						m_SFXM->SFX_play("sfx/cursormove", m_sfx_volume);
						break;
				}
			}
			if (keyPress(KKEY_RIGHT,-1,k)) {
				switch(m_menu[m_title_menu_selected]->m_ID) {
				case 102: // change music volume up
					if (m_music_volume==MIX_MAX_VOLUME) {
						m_music_volume=0;
					} else {
						m_music_volume+=32;
						if (m_music_volume>MIX_MAX_VOLUME) m_music_volume = MIX_MAX_VOLUME;
					}
					save_configuration();
					getMenu(MENU_OPTIONS);
					m_SFXM->SFX_play("sfx/cursormove", m_sfx_volume);
					m_MusicM->music_volume(0, m_music_volume);
					break;
				case 103: // change sfx volume up
					if (m_sfx_volume==MIX_MAX_VOLUME) {
						m_sfx_volume=0;
					} else {
						m_sfx_volume+=32;
						if (m_sfx_volume>MIX_MAX_VOLUME) m_music_volume = MIX_MAX_VOLUME;
					}
					save_configuration();
					getMenu(MENU_OPTIONS);
					m_SFXM->SFX_play("sfx/cursormove", m_sfx_volume);
					break;
				}
			}
			if (keyPress(KKEY_FIRE,-1,k)) {
				switch(m_menu[m_title_menu_selected]->m_ID) {
				case 0:	// Start a 1 player game
						m_title_menu_action = 0;
						m_title_menu_state = 2;
						break;
				case 1: // Start a 2 player game
						m_title_menu_action = 1;
						m_title_menu_state = 2;
						break;
				case 2: // Go to how to play
						m_title_menu_action = 2;
						m_title_menu_state = 2;
						break;
				case 3: // Highscores
						m_title_menu_action = 3;
						m_title_menu_state = 2;
						break;
				case 4: // Go to Options menu
						m_title_menu_action = 4;
						m_title_menu_state = 2;
						break;
				case 5: // Quit
						m_title_menu_action = 5;
						m_title_menu_state = 2;
						break;

				case 100: // Go to player 1 controls
						m_title_menu_action = 100;
						m_title_menu_state = 2;
						break;
				case 101: // Back to player 2 controls
						m_title_menu_action = 101;
						m_title_menu_state = 2;
						break;
				case 102: // change music volume up
						if (m_music_volume==MIX_MAX_VOLUME) {
							m_music_volume=0;
						} else {
							m_music_volume+=32;
							if (m_music_volume>MIX_MAX_VOLUME) m_music_volume = MIX_MAX_VOLUME;
						}
						save_configuration();
						getMenu(MENU_OPTIONS);
						m_MusicM->music_volume(0, m_music_volume);
						break;
				case 103: // change sfx volume up
						if (m_sfx_volume==MIX_MAX_VOLUME) {
							m_sfx_volume=0;
						} else {
							m_sfx_volume+=32;
							if (m_sfx_volume>MIX_MAX_VOLUME) m_music_volume = MIX_MAX_VOLUME;
						}
						save_configuration();
						getMenu(MENU_OPTIONS);
						break;
						
				case 104: // toogle fullscreen/windowed
                        fullscreen = (fullscreen ? false : true);
                        screen_sfc = toogle_video_mode(fullscreen);
                        save_configuration();
						getMenu(MENU_OPTIONS);
						break;
				case 105: // Back to main
						m_title_menu_action = 105;
						m_title_menu_state = 2;
						break;

				case 200:
						// Change joystick for player 1:
						m_joystick1_num++;
						if (m_joystick1_num>=k->n_joysticks) m_joystick1_num=-1;
						getMenu(MENU_INPUT1);
						break;
				case 300:
						// Change joystick for player 2:
						m_joystick2_num++;
						if (m_joystick2_num>=k->n_joysticks) m_joystick2_num=-1;
						getMenu(MENU_INPUT2);
						break;

				case 201:
						m_title_menu_redefining_key_player = 1;
						m_title_menu_redefining_key = KKEY_UP;
						getMenu(MENU_INPUT1);
						break;
				case 202:
						m_title_menu_redefining_key_player = 1;
						m_title_menu_redefining_key = KKEY_DOWN;
						getMenu(MENU_INPUT1);
						break;
				case 203:
						m_title_menu_redefining_key_player = 1;
						m_title_menu_redefining_key = KKEY_LEFT;
						getMenu(MENU_INPUT1);
						break;
				case 204:
						m_title_menu_redefining_key_player = 1;
						m_title_menu_redefining_key = KKEY_RIGHT;
						getMenu(MENU_INPUT1);
						break;
				case 205:
						m_title_menu_redefining_key_player = 1;
						m_title_menu_redefining_key = KKEY_FIRE;
						getMenu(MENU_INPUT1);
						break;
				case 206:
						m_title_menu_redefining_key_player = 1;
						m_title_menu_redefining_key = KKEY_QUIT;
						getMenu(MENU_INPUT1);
						break;

				case 301:
						m_title_menu_redefining_key_player = 2;
						m_title_menu_redefining_key = KKEY_UP;
						getMenu(MENU_INPUT2);
						break;
				case 302:
						m_title_menu_redefining_key_player = 2;
						m_title_menu_redefining_key = KKEY_DOWN;
						getMenu(MENU_INPUT2);
						break;
				case 303:
						m_title_menu_redefining_key_player = 2;
						m_title_menu_redefining_key = KKEY_LEFT;
						getMenu(MENU_INPUT2);
						break;
				case 304:
						m_title_menu_redefining_key_player = 2;
						m_title_menu_redefining_key = KKEY_RIGHT;
						getMenu(MENU_INPUT2);
						break;
				case 305:
						m_title_menu_redefining_key_player = 2;
						m_title_menu_redefining_key = KKEY_FIRE;
						getMenu(MENU_INPUT2);
						break;
				case 306:
						m_title_menu_redefining_key_player = 2;
						m_title_menu_redefining_key = KKEY_QUIT;
						getMenu(MENU_INPUT2);
						break;

				case 208:
				case 308: // Back to Options
						m_title_menu_action = 208;
						m_title_menu_state = 2;
						break;
				}
				m_SFXM->SFX_play("sfx/cursorselect", m_sfx_volume);
			}
		}
	}

    return STATE_TITLE;
}

void KnightmareApp::getMenu(int menuID)
{
	char tmp[128];

	m_menu.Delete();
	if (m_title_menu_name!=0) delete m_title_menu_name;
	m_title_menu_name = 0;

	m_title_menu_ID = menuID;

	switch(menuID) {
	case 0: m_title_menu_name = new Symbol("");
			m_menu.Add(new MenuItem("Start 1 Player",0));
			m_menu.Add(new MenuItem("Start 2 Players",1));
			m_menu.Add(new MenuItem("How to play",2));
			m_menu.Add(new MenuItem("Highscores",3));
			m_menu.Add(new MenuItem("Options",4));
			m_menu.Add(new MenuItem("Quit",5));
			break;

	case 1: m_title_menu_name = new Symbol("Options");
			m_menu.Add(new MenuItem("Player 1 Input",100));
			m_menu.Add(new MenuItem("Player 2 Input",101));

			{
				if (m_music_volume==0) {
					sprintf(tmp,"Music Volume: silent");
				} else if (m_music_volume<33) {
					sprintf(tmp,"Music Volume: low");
				} else if (m_music_volume<65) {
					sprintf(tmp,"Music Volume: medium");
				} else if (m_music_volume<97) {
					sprintf(tmp,"Music Volume: high");
				} else {
					sprintf(tmp,"Music Volume: max");
				}
				m_menu.Add(new MenuItem(tmp,102));
			}
			{
				if (m_sfx_volume==0) {
					sprintf(tmp,"SFX Volume: silent");
				} else if (m_sfx_volume<33) {
					sprintf(tmp,"SFX Volume: low");
				} else if (m_sfx_volume<65) {
					sprintf(tmp,"SFX Volume: medium");
				} else if (m_sfx_volume<97) {
					sprintf(tmp,"SFX Volume: high");
				} else {
					sprintf(tmp,"SFX Volume: max");
				}
				m_menu.Add(new MenuItem(tmp,103));
			}

			if (fullscreen) m_menu.Add(new MenuItem("Full Screen: on",104));
					   else m_menu.Add(new MenuItem("Full Screen: off",104));
			m_menu.Add(new MenuItem("Back",105));
			break;

	case 2: m_title_menu_name = new Symbol("Input Player 1");
			if (m_joystick1_num==-1) {
				m_menu.Add(new MenuItem("Joystick: -",200));
			} else {
				sprintf(tmp,"Joystick: %i",m_joystick1_num+1);
				m_menu.Add(new MenuItem(tmp,200));
			}
			{
				if (m_title_menu_redefining_key_player==1 && m_title_menu_redefining_key==KKEY_UP) 
					sprintf(tmp,"Up: - choose a key -");
				else
					sprintf(tmp,"Up: %s",SDL_GetKeyName((SDLKey)m_keys_configuration1[KKEY_UP]));
				m_menu.Add(new MenuItem(tmp,201));
				if (m_title_menu_redefining_key_player==1 && m_title_menu_redefining_key==KKEY_DOWN) 
					sprintf(tmp,"Down: - choose a key -");
				else
					sprintf(tmp,"Down: %s",SDL_GetKeyName((SDLKey)m_keys_configuration1[KKEY_DOWN]));
				m_menu.Add(new MenuItem(tmp,202));
				if (m_title_menu_redefining_key_player==1 && m_title_menu_redefining_key==KKEY_LEFT) 
					sprintf(tmp,"Left: - choose a key -");
				else
					sprintf(tmp,"Left: %s",SDL_GetKeyName((SDLKey)m_keys_configuration1[KKEY_LEFT]));
				m_menu.Add(new MenuItem(tmp,203));
				if (m_title_menu_redefining_key_player==1 && m_title_menu_redefining_key==KKEY_RIGHT) 
					sprintf(tmp,"Right: - choose a key -");
				else
					sprintf(tmp,"Right: %s",SDL_GetKeyName((SDLKey)m_keys_configuration1[KKEY_RIGHT]));
				m_menu.Add(new MenuItem(tmp,204));
				if (m_title_menu_redefining_key_player==1 && m_title_menu_redefining_key==KKEY_FIRE) 
					sprintf(tmp,"Fire: - choose a key -");
				else
					sprintf(tmp,"Fire: %s",SDL_GetKeyName((SDLKey)m_keys_configuration1[KKEY_FIRE]));
				m_menu.Add(new MenuItem(tmp,205));
				if (m_title_menu_redefining_key_player==1 && m_title_menu_redefining_key==KKEY_QUIT) 
					sprintf(tmp,"Quit: - choose a key -");
				else
					sprintf(tmp,"Quit: %s",SDL_GetKeyName((SDLKey)m_keys_configuration1[KKEY_QUIT]));
				m_menu.Add(new MenuItem(tmp,206));
			}
			m_menu.Add(new MenuItem("Back",208));
			break;

	case 3: m_title_menu_name = new Symbol("Input Player 2");
			if (m_joystick2_num==-1) {
				m_menu.Add(new MenuItem("Joystick: -",200));
			} else {
				sprintf(tmp,"Joystick: %i",m_joystick2_num+1);
				m_menu.Add(new MenuItem(tmp,200));
			}
			{
				if (m_title_menu_redefining_key_player==2 && m_title_menu_redefining_key==KKEY_UP) 
					sprintf(tmp,"Up: - choose a key -");
				else
					sprintf(tmp,"Up: %s",SDL_GetKeyName((SDLKey)m_keys_configuration2[KKEY_UP]));
				m_menu.Add(new MenuItem(tmp,301));
				if (m_title_menu_redefining_key_player==2 && m_title_menu_redefining_key==KKEY_DOWN) 
					sprintf(tmp,"Down: - choose a key -");
				else
					sprintf(tmp,"Down: %s",SDL_GetKeyName((SDLKey)m_keys_configuration2[KKEY_DOWN]));
				m_menu.Add(new MenuItem(tmp,302));
				if (m_title_menu_redefining_key_player==2 && m_title_menu_redefining_key==KKEY_LEFT) 
					sprintf(tmp,"Left: - choose a key -");
				else
					sprintf(tmp,"Left: %s",SDL_GetKeyName((SDLKey)m_keys_configuration2[KKEY_LEFT]));
				m_menu.Add(new MenuItem(tmp,303));
				if (m_title_menu_redefining_key_player==2 && m_title_menu_redefining_key==KKEY_RIGHT) 
					sprintf(tmp,"Right: - choose a key -");
				else
					sprintf(tmp,"Right: %s",SDL_GetKeyName((SDLKey)m_keys_configuration2[KKEY_RIGHT]));
				m_menu.Add(new MenuItem(tmp,304));
				if (m_title_menu_redefining_key_player==2 && m_title_menu_redefining_key==KKEY_FIRE) 
					sprintf(tmp,"Fire: - choose a key -");
				else
					sprintf(tmp,"Fire: %s",SDL_GetKeyName((SDLKey)m_keys_configuration2[KKEY_FIRE]));
				m_menu.Add(new MenuItem(tmp,305));
				if (m_title_menu_redefining_key_player==2 && m_title_menu_redefining_key==KKEY_QUIT) 
					sprintf(tmp,"Quit: - choose a key -");
				else
					sprintf(tmp,"Quit: %s",SDL_GetKeyName((SDLKey)m_keys_configuration2[KKEY_QUIT]));
				m_menu.Add(new MenuItem(tmp,306));
			}
			m_menu.Add(new MenuItem("Back",308));
			break;
	}
}


void KnightmareApp::title_draw(void)
{
    glClearColor(0, 0, 0, 1);
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glTranslatef(320,140,0);
	if (m_title_menu_state==2) {
		m_GLTM->get("kmlogo1")->draw(1,1,1,this->m_title_entry_effect,0,0,0,0,this->m_title_entry_effect);
		m_GLTM->get("kmlogo2")->draw(1,1,1,this->m_title_entry_effect,0,0,0,0,this->m_title_entry_effect);
	} else {
		m_GLTM->get("kmlogo1")->draw(1,1,1,1,0,0,0,0,1);
		m_GLTM->get("kmlogo2")->draw(1,1,1,1,0,0,0,0,1);
	}
	glPopMatrix();

	drawMenu(this->m_title_entry_effect);

}


void KnightmareApp::drawMenu(float enterEffect) {
	// TODO: some fancy effect here, for now just fade in/out
	float y = 300;
	int i = 0;
	MenuItem *item;

	m_menu.Rewind();
	while(m_menu.Iterate(item)) {

		if (i==m_title_menu_selected) {
			float f = float(sin(m_state_cycle*0.25))*0.5f + 0.5f;
			Binterface::print_center(item->m_text->get(),m_font16,320,y,enterEffect*enterEffect*f,enterEffect*enterEffect*f,enterEffect,enterEffect);
		} else {
			Binterface::print_center(item->m_text->get(),m_font16,320,y,enterEffect*enterEffect,enterEffect*enterEffect,enterEffect,enterEffect);
		}

		y+=20;
		i++;
	}


}
